#!/usr/bin/env python

from abc import ABCMeta, abstractmethod
import logging
import os

import dxpy
import requests

from soyuz.data.storages import DNAnexusStorage, ExodusStorage
from soyuz.dx.variables import Type, Property
from soyuz import __version__ as version
from soyuz.utils import UploaderException


class Record(object):
    __metaclass__ = ABCMeta

    def __init__(self, storage, basedir, name):
        self.storage = storage
        self.__id = None
        self.__details = None
        self.__tags = None
        self.__types = None
        self.__properties = None
        self.__record = None
        self.__basedir = basedir
        self.__name = name

    @abstractmethod
    def get_id(self):
        raise NotImplementedError()

    @abstractmethod
    def close(self):
        raise NotImplementedError()

    @abstractmethod
    def get_details(self):
        raise NotImplementedError()

    @abstractmethod
    def add_tags(self, tags):
        raise NotImplementedError()

    @abstractmethod
    def set_details(self, details):
        raise NotImplementedError()


class DNAnexusRecord(Record):
    def __init__(self, storage, basedir, name):
        super(DNAnexusRecord, self).__init__(storage, basedir, name)
        self.__record = dxpy.new_dxrecord(types=[Type.UPLOAD_SENTINEL],
                                          folder=os.path.join(basedir, name).replace("\\", "/"),
                                          name="{}_upload_sentinel".format(name),
                                          properties={Property.RUN_FOLDER: name,
                                                      Property.VERSION: version},
                                          parents=True)

    def get_id(self):
        return self.__record.get_id()

    def close(self):
        return self.__record.close()

    def get_details(self):
        return self.__record.get_details()

    def set_details(self, details):
        return self.__record.set_details(details)

    def add_tags(self, tags):
        return self.__record.add_tags(tags)


class ExodusRecord(Record):
    def __init__(self, storage, basedir, name):
        super(ExodusRecord, self).__init__(storage, basedir, name)
        self.__id = None
        self.__record = None
        self.__name = name
        self.__basedir = basedir
        self.__details = {}
        self.__properties = {}
        self.__tags = []
        self.__types = []
        self.exodus_url = self.storage.exodus_url

    def close(self):
        data = {'name': self.__name,
                'folder': os.path.join(self.__basedir, self.__name).replace("\\", "/"),
                'types': [Type.UPLOAD_SENTINEL],
                'properties': {Property.RUN_FOLDER: self.__name,
                               Property.VERSION: version},
                'tags': self.__tags,
                'details': self.__details
                }
        response = requests.post(os.path.join(self.exodus_url, "2.0/data/records/new"), json=data)
        response.raise_for_status()
        data = response.json()
        self.__id = data['link']
        logging.info("Successfully created: {}".format(os.path.join(self.__basedir, self.__name)))
        return self.get_id()

    def get_id(self):
        return self.__id

    def get_details(self):
        return self.__details

    def set_details(self, details):
        self.__details = details

    def add_tags(self, tags):
        self.__tags = tags


class RecordFactory(object):
    @staticmethod
    def create(storage, basedir, name):
        if isinstance(storage, DNAnexusStorage):
            return DNAnexusRecord(storage, basedir, name)
        if isinstance(storage, ExodusStorage):
            return ExodusRecord(storage, basedir, name)
        raise UploaderException("Storage with name {} was not found".format(name))



#!/usr/bin/env python

from abc import ABCMeta, abstractmethod

from soyuz.data.records import RecordFactory, DNAnexusRecord, ExodusRecord
from soyuz.dx.variables import Type


class SentinelBase(object):
    __metaclass__ = ABCMeta

    DATA_KEY = "data"
    FASTQ_KEY = "fastq"
    METRICS_KEY = "run_metrics"
    DX_LINK_KEY = "$dnanexus_link"

    def __init__(self, storage, basedir, name):
        self.record = RecordFactory.create(storage, basedir, name)

    @abstractmethod
    def add_file(self, data_file, file_id):
        raise NotImplementedError()

    def add_link(self, file_id):
        if isinstance(self.record, DNAnexusRecord):
            return {SentinelBase.DX_LINK_KEY: file_id}
        elif isinstance(self.record, ExodusRecord):
            return file_id

    def get_id(self):
        return self.record.get_id()

    def close(self):
        self.record.close()


class WesSignateraSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        details = self.record.get_details()
        sample_id = data_file.get_sample_id()
        if sample_id:
            self.record.add_tags([sample_id])
            self.__add_data_file_details(details, sample_id, self.add_link(file_id))
        else:
            self.__add_metrics_details(details, self.add_link(file_id))
        self.record.set_details(details)

    @staticmethod
    def __add_data_file_details(details, sample_id, file_link):
        if SentinelBase.DATA_KEY not in details:
            details[SentinelBase.DATA_KEY] = {}
        data_details = details[SentinelBase.DATA_KEY]
        if sample_id not in data_details:
            data_details[sample_id] = []
        data_details[sample_id].append(file_link)

    @staticmethod
    def __add_metrics_details(details, file_link):
        if SentinelBase.METRICS_KEY not in details:
            details[SentinelBase.METRICS_KEY] = []
        details[SentinelBase.METRICS_KEY].append(file_link)


class PersonalisSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        details = self.record.get_details()
        sample_id = data_file.get_sample_id()
        if sample_id:
            self.record.add_tags([sample_id])
            if data_file.get_type() == Type.BAM:
                self.__add_data_file_details(details, sample_id, self.add_link(file_id))
            elif data_file.get_type() == Type.FASTQ:
                self.__add_fastq_file_details(details, sample_id, self.add_link(file_id))
        else:
            self.__add_metrics_details(details, self.add_link(file_id))
        self.record.set_details(details)

    @staticmethod
    def __add_data_file_details(details, sample_id, file_link):
        if SentinelBase.DATA_KEY not in details:
            details[SentinelBase.DATA_KEY] = {}
        data_details = details[SentinelBase.DATA_KEY]
        if sample_id not in data_details:
            data_details[sample_id] = []
        data_details[sample_id].append(file_link)

    @staticmethod
    def __add_fastq_file_details(details, sample_id, file_link):
        if SentinelBase.FASTQ_KEY not in details:
            details[SentinelBase.FASTQ_KEY] = {}
        data_details = details[SentinelBase.FASTQ_KEY]
        if sample_id not in data_details:
            data_details[sample_id] = []
        data_details[sample_id].append(file_link)

    @staticmethod
    def __add_metrics_details(details, file_link):
        if SentinelBase.METRICS_KEY not in details:
            details[SentinelBase.METRICS_KEY] = []
        details[SentinelBase.METRICS_KEY].append(file_link)


class RawSentinel(SentinelBase):
    def add_file(self, data_file, file_id):
        pass



#!/usr/bin/env python
from setuptools import setup, find_packages
from codecs import open
from os import path
import soyuz
import soyuz.dx
import soyuz.data

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='natera-data-uploader',
    version=soyuz.__version__,
    description='Natera Uploader',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Nikita Berezin',
    author_email='nberezin@natera.com',
    packages=find_packages(exclude=["*.tests", "tests.*", "tests"]),
    entry_points={
        'console_scripts': [
            'uploader=soyuz.entrypoint:main',
        ],
    },
    install_requires=[
        'dxpy>=0.288.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 2'
    ],
    python_requires='>=2.7'
)
